/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.task.field;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Point;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import nl.cloudfarming.client.field.model.ActivityField;
import nl.cloudfarming.client.geoviewer.Geographical;
import nl.cloudfarming.client.model.ItemIdType;
import nl.cloudfarming.client.task.ExportableTask;
import org.geotools.coverage.grid.GridCoverage2D;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 * @author Maciek Dulkiewicz
 */
public class FieldTask extends ExportableTask {

    public static final double MIN_WORK_WIDTH = 5;
    private double workWidth = MIN_WORK_WIDTH;
    public static final String MIME_TYPE = "application/x-agrosense-field-task";
    public static final String EXT = "ftk";
    public static final String PROP_TREATMENTZONE_VALUE = "treatmentZoneValue";

    public FieldTask(ActivityField activityField) {
        super(activityField.getURI(), new Class[]{ActivityField.class}, ItemIdType.FTK);
        ActivityField copy = new ActivityField(activityField);
        getContext().addItem(copy);
        setName(activityField.getName());
        
        ic.add(new FieldTaskGeometrical(activityField));
    }

    @Override
    public double getWorkWidth() {
        return workWidth;
    }

    public void setWorkWidth(double workWidth) {
        double oldWorkWidth = this.workWidth;
        this.workWidth = workWidth;
        setRegenerateGrid(true);
        getPropertyChangeSupport().firePropertyChange(ExportableTask.PROP_WORK_WIDTH, oldWorkWidth, this.workWidth);
    }

    public void fireExternalPropertyChange(String eventName, Object oldValue, Object newValue) {
        getPropertyChangeSupport().firePropertyChange(eventName, oldValue, newValue);
    }

    
    
    protected class FieldTaskGeometrical implements Geographical<GridCoverage2D>, Serializable {

        private ActivityField activityField;

        public FieldTaskGeometrical(ActivityField activityField) {
            this.activityField = activityField;
        }

        @Override
        public Point getCentroid() {
            return activityField.getCentroid();
        }

        @Override
        public Geometry getBoundingBox() {
            return activityField.getBoundingBox();
        }

        @Override
        public GridCoverage2D getRenderObject(Envelope envelope) {
            return FieldTask.this.getExportPreviewGrid();
        }

        @Override
        public String getTooltipText() {
            return activityField.getTooltipText();
        }

        @Override
        public String getIconPath() {
            return activityField.getIconPath();
        }

        @Override
        public String getIconLabel() {
            return activityField.getIconLabel();
        }

        @Override
        public void addPropertyChangeListener(PropertyChangeListener listener) {
            activityField.addPropertyChangeListener(listener);
        }

        @Override
        public void removePropertyChangeListener(PropertyChangeListener listener) {
            activityField.removePropertyChangeListener(listener);
        }
    }
}
